/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_COMMON_CONFIGURATION_H_
#define DBUG_COMMON_CONFIGURATION_H_ 1

#include <libconfig.h++>
#include <string>

#include "dbug/common/logging.h"
#include "dbug/common/macros.h"
#include "dbug/common/system.h"

using libconfig::Config;

namespace dbug {

class Configuration {
 public:
  static void Initialize();
  static void Finalize();

  static std::string arbiter_hostname();
  static int arbiter_port();
  static System::TimeSpec arbiter_timeout();
  static Logging::Verbosity arbiter_verbosity();
  static std::string explorer_hostname();
  static int explorer_port();
  static Logging::Verbosity explorer_verbosity();
  static Logging::Verbosity interposition_verbosity();
  static int max_backtrace_length();
  static int max_interfaces();
  static int random_state_length();
  static Logging::Verbosity verbosity();

 private:
  static Config *config_;

  DISALLOW_ALL_DEFAULT(Configuration);
};

}  // namespace dbug

#endif  // DBUG_COMMON_CONFIGURATION_H_
