/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_COMMON_LOCK_H_
#define DBUG_COMMON_LOCK_H_ 1

#include "dbug/common/macros.h"
#include "dbug/common/system.h"

namespace dbug {

class MutexLock {
 public:
  explicit MutexLock(System::Mutex *mutex);
  ~MutexLock();

 private:
  System::Mutex *mutex_;

  DISALLOW_COPY_AND_ASSIGN(MutexLock);
};

class ReadLock {
 public:
  explicit ReadLock(System::RWLock *rwlock);
  ~ReadLock();

 private:
  System::RWLock *rwlock_;

  DISALLOW_COPY_AND_ASSIGN(ReadLock);
};

class WriteLock {
 public:
  explicit WriteLock(System::RWLock *rwlock);
  ~WriteLock();

 private:
  System::RWLock *rwlock_;

  DISALLOW_COPY_AND_ASSIGN(WriteLock);
};

}  // namespace dbug

#endif  // DBUG_COMMON_LOCK_H_

