/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_COMMON_MACROS_H_
#define DBUG_COMMON_MACROS_H_ 1

#define DBUG_CONSTRUCTOR __attribute__ ((constructor))
#define DBUG_DESTRUCTOR __attribute__ ((destructor))
#define DBUG_NORETURN __attribute__ ((noreturn))
#define DBUG_PUBLIC __attribute__ ((visibility("default")))
#define DBUG_RESTRICT __restrict
#define DBUG_UNUSED __attribute__ ((unused))

// BEGIN_C_DECLS should be used at the beginning of your declarations,
// so that C++ compilers don't mangle their names. Use END_C_DECLS at
// the end of C declarations.
#undef BEGIN_C_DECLS
#undef END_C_DECLS
#ifdef __cplusplus
#  define BEGIN_C_DECLS extern "C" {
#  define END_C_DECLS }
#else
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif

#define DBUG_REQUIRE_SYMBOL(var, type, name)  do {              \
    if (var == NULL) {                                          \
      const char *err_str;                                      \
      dlerror();                                                \
      var = reinterpret_cast<type>(dlsym(RTLD_NEXT, (name)));   \
      err_str = dlerror();                                      \
      if (var == NULL) {                                        \
        ERROR("dlsym(%s) failed: %s", (name), err_str);         \
      }                                                         \
    }                                                           \
  } while (0)

#define DISALLOW_COPY_AND_ASSIGN(type)          \
  type(const type&);                            \
  void operator=(const type&)

#define DISALLOW_ALL_DEFAULT(type)              \
  type();                                       \
  DISALLOW_COPY_AND_ASSIGN(type)

#endif  // DBUG_COMMON_MACROS_H_
