/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "dbug/arbiter/handler.h"

#include "dbug/arbiter/logging.h"
#include "dbug/arbiter/server.h"
#include "dbug/arbiter/thread.h"

namespace dbug {
namespace arbiter {

void Handler::MPIInit(const int tid,
                      const int any_source,
                      const int any_tag,
                      const int null_process,
                      const int rank) {
  Server::Lock lock(server_);
  server_->mpi()->set_any_source(any_source);
  server_->mpi()->set_any_tag(any_tag);
  server_->mpi()->set_rank(any_source, any_source);
  server_->mpi()->set_rank(null_process, null_process);
  server_->mpi()->set_rank(rank, tid - 1);
}

int Handler::MPIRank(const int rank) {
  Server::Lock lock(server_);
  return server_->mpi()->rank(rank);
}

void Handler::MPIRequestComplete(const int tid,
                                 const std::string &request_id) {
  Server::Lock lock(server_);
  Thread *thread = server_->thread_lookup(tid);
  if (thread == NULL) {
    WARNING("Thread %u does not exists.", tid);
  } else {
    server_->mpi()->add_request(request_id, thread->clock());
  }
}

}  // namespace arbiter
}  // namespace dbug
