/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "dbug/common/clock.h"

#include <vector>

namespace dbug {

void Clock::JoinWith(const std::vector<int> &other) {
  int i = 0, n1 = this->size(), n2 = other.size();
  for (; i < n1 && i < n2; i++) {
    this->at(i) = (this->at(i) < other[i] ? other[i] : this->at(i));
  }
  for (; i < n2; i++) { this->push_back(other[i]); }
}

Clock::Order Clock::CompareWith(const std::vector<int> &other) {
  int n1 = this->size(), n2 = other.size();
  // <after> evalutes if this >= other.
  bool after = (n1 >= n2);
  // <before> evaluates if this <= other.
  bool before = (n1 <= n2);
  for (int i = 0; i < n1 && i < n2; i++) {
    if (this->at(i) < other[i]) { after = false; }
    if (other[i] < this->at(i)) { before = false; }
  }
  if (before && after) { return Clock::EQUAL; }
  if (before && !after) { return Clock::BEFORE; }
  if (!before && after) { return Clock::AFTER; }
  return Clock::CONCURRENT;
}

}  // namespace dbug
