/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <map>

#include "dbug/common/system.h"
#include "dbug/common/util.h"
#include "dbug/interposition/handler.h"
#include "dbug/interposition/logging.h"
#include "dbug/interposition/original.h"
#include "src/interposition/gen/pthread-keys.cc"

namespace dbug {
namespace interposition {

#ifdef DBUG_HAVE_PTHREAD_KEY_CREATE
int Handler::pthread_key_create(pthread_key_t * key,
                                void (*destructor)(void *)) {  // NOLINT
  switch (arbiter_->Choose(arbiter_->ThreadID(),
                           "pthread_key_create",
                           Util::GetBacktrace(),
                           3)) {
    case 1:
      // Simulate the lack of system resources.
      errno = EAGAIN;
      return EAGAIN;
    case 2:
      // Simulate the lack of memory.
      errno = ENOMEM;
      return ENOMEM;
    default:
      return Original::pthread_key_create(key, destructor);
  }
}
#endif

#ifdef DBUG_HAVE_PTHREAD_SETSPECIFIC
int Handler::pthread_setspecific(pthread_key_t key, const void *value) {
  switch (arbiter_->Choose(arbiter_->ThreadID(),
                           "pthread_setspecific",
                           Util::GetBacktrace(),
                           2)) {
    case 1:
      // Simulate the lack of memory.
      errno = ENOMEM;
      return ENOMEM;
    default:
      return Original::pthread_setspecific(key, value);
  }
}
#endif

}  // namespace interposition
}  // namespace dbug
