/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

// TODO(jsimsa): Add support for all non-reentrant POSIX functions.

#include "dbug/common/util.h"
#include "dbug/interposition/handler.h"
#include "dbug/interposition/logging.h"
#include "dbug/interposition/original.h"
#include "src/interposition/gen/reentrant.cc"

namespace dbug {
namespace interposition {

#if DBUG_HAVE_GETHOSTBYADDR
struct hostent *Handler::gethostbyaddr(const void *addr,
                                       socklen_t length,
                                       int type) {
  arbiter_->NonReentrantFunction(arbiter_->ThreadID(),
                                 "gethostbyaddr",
                                 Util::GetBacktrace());
  return Original::gethostbyaddr(addr, length, type);
}
#endif

#if DBUG_HAVE_GETHOSTBYADDR
struct hostent *Handler::gethostbyname(const char *name) {
  arbiter_->NonReentrantFunction(arbiter_->ThreadID(),
                                 "gethostbyname",
                                 Util::GetBacktrace());
  return Original::gethostbyname(name);
}
#endif

#if DBUG_HAVE_INET_NTOA
char * Handler::inet_ntoa(struct in_addr in) {
  arbiter_->NonReentrantFunction(arbiter_->ThreadID(),
                                 "inet_ntoa",
                                 Util::GetBacktrace());
  return Original::inet_ntoa(in);
}
#endif

#if DBUG_HAVE_STRTOK
char *Handler::strtok(char *DBUG_RESTRICT str,  // NOLINT
                      const char *DBUG_RESTRICT delim) {
  arbiter_->NonReentrantFunction(arbiter_->ThreadID(),
                                 "strtok",
                                 Util::GetBacktrace());
  return Original::strtok(str, delim);  // NOLINT
}
#endif

}  // namespace interposition
}  // namespace dbug
