/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <mpi.h>
#include <stdio.h>

#include <boost/scoped_array.hpp>
#include <sstream>

int main(int argc, char *argv[]) {
  MPI_Init(&argc, &argv);
  int myrank;
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  int size;
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  boost::scoped_array<int> send_buffer(new int[size]);
  boost::scoped_array<int> send_counts(new int[size]);
  boost::scoped_array<int> sdispls(new int[size]);
  boost::scoped_array<int> recv_counts(new int[size]);
  boost::scoped_array<int> rdispls(new int[size]);
  for (int i = 0; i < size; i++) {
    send_buffer[i] = myrank + 1;
    send_counts[i] = i + 1;
    sdispls[i] = 0;
    recv_counts[i] = myrank + 1;
    rdispls[i] = i * (myrank + 1);
  }
  boost::scoped_array<int> recv_buffer(new int[(myrank + 1) * size]);
  MPI_Alltoallv(send_buffer.get(), send_counts.get(), sdispls.get(), MPI_INT,
                recv_buffer.get(), recv_counts.get(), rdispls.get(), MPI_INT,
                MPI_COMM_WORLD);
  std::stringstream stream;
  stream << "Process " << myrank << " received:";
  for (int i = 0; i < (size * (myrank + 1)); i++) {
    stream << " " << recv_buffer[i];
  }
  printf("%s.\n", stream.str().c_str());
  MPI_Finalize();
  return 0;
}
