/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <mpi.h>
#include <stdio.h>
#include <string.h>

#include <boost/scoped_array.hpp>

#define BUFFER_SIZE 256

int main(int argc, char *argv[]) {
  char message[BUFFER_SIZE];
  int myrank;
  MPI_Status status;
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  if (myrank == 0) {
    /* code for process zero */
    int size;
    MPI_Pack_size(BUFFER_SIZE, MPI_CHAR, MPI_COMM_WORLD, &size);
    int bufsize = MPI_BSEND_OVERHEAD + size;
    boost::scoped_array<char> buffer(new char[bufsize]);
    MPI_Buffer_attach(buffer.get(), bufsize);
    snprintf(message, BUFFER_SIZE, "Message A");
    MPI_Bsend(message, strlen(message) + 1, MPI_CHAR, 1, 99, MPI_COMM_WORLD);
    printf("Process %d sent '%s'.\n", myrank, message);
    int idummy;
    char *dummy;
    MPI_Buffer_detach(&dummy, &idummy);
  } else if (myrank == 1) {
    /* code for process one */
    MPI_Recv(message, BUFFER_SIZE, MPI_CHAR, 0, 99, MPI_COMM_WORLD, &status);
    printf("Process %d received '%s'.\n", myrank, message);
  }
  MPI_Finalize();
  return 0;
}
