/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <mpi.h>
#include <stdio.h>

#include <boost/scoped_array.hpp>

int main(int argc, char *argv[]) {
  MPI_Init(&argc, &argv);
  int myrank;
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  int size;
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  boost::scoped_array<int> displs(new int[size]);
  boost::scoped_array<int> recv_counts(new int[size]);
  boost::scoped_array<int> send_buffer(new int[myrank + 1]);
  for (int i = 0; i < myrank + 1; i++) { send_buffer[i] = i + 1; }
  int rsize = 0;
  for (int i = 0; i < size; i++) {
    recv_counts[i] = i + 1;
    displs[i] = rsize;
    rsize += i + 1;
  }
  boost::scoped_array<int> recv_buffer(new int[rsize]);
  for (int i = 0; i < rsize; i++) { recv_buffer[i] = 0; }
  MPI_Gatherv(send_buffer.get(), myrank + 1, MPI_INT,
              recv_buffer.get(), recv_counts.get(), displs.get(), MPI_INT,
              0, MPI_COMM_WORLD);
  if (myrank == 0) {
    printf("Process %d received: ", myrank);
    for (int i = 0; i < rsize; i++) {
      if (i > 0) { printf(" "); }
      printf("%d", recv_buffer[i]);
    }
    printf(".\n");
  }
  MPI_Finalize();
  return 0;
}
