/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <mpi.h>
#include <stdio.h>

#include <boost/scoped_array.hpp>
#include <sstream>

int main(int argc, char *argv[]) {
  MPI_Init(&argc, &argv);
  int myrank;
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  int size;
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  int ssize = (size + 1) * (size / 2);
  if (size % 2 == 1) { ssize += (size + 1) / 2; }
  boost::scoped_array<int> send_buffer(new int[ssize]);
  unsigned seed = myrank;
  for (int i = 0; i < ssize; i++) { send_buffer[i] = rand_r(&seed) % 256; }
  boost::scoped_array<int> recv_buffer(new int[myrank + 1]);
  boost::scoped_array<int> recv_counts(new int[size]);
  for (int i = 0; i < size; i++) { recv_counts[i] = i + 1; }
  std::stringstream send_stream;
  send_stream << "Process " << myrank << " sends:";
  for (int i = 0; i < ssize; i++) {
    send_stream << " ";
    send_stream.width(3);
    send_stream << send_buffer[i];
  }
  printf("%s.\n", send_stream.str().c_str());
  MPI_Reduce_scatter(send_buffer.get(), recv_buffer.get(), recv_counts.get(),
                     MPI_INT, MPI_MAX, MPI_COMM_WORLD);
  std::stringstream recv_stream;
  recv_stream << "Process " << myrank << " received:";
  for (int i = 0; i <= myrank; i++) {
    recv_stream << " ";
    recv_stream.width(3);
    recv_stream << recv_buffer[i];
  }
  printf("%s.\n", recv_stream.str().c_str());
  MPI_Finalize();
  return 0;
}
