/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <mpi.h>
#include <stdio.h>
#include <string.h>

#include <boost/scoped_array.hpp>

#define BUFFER_SIZE 256

int main(int argc, char *argv[]) {
  int myrank, size;
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  if (myrank == 0) {
    boost::scoped_array<char *> messages(new char *[size - 1]());
    boost::scoped_array<MPI_Request> requests(new MPI_Request[size - 1]());
    boost::scoped_array<MPI_Status> statuses(new MPI_Status[size - 1]());
    for (int i = 0; i < size - 1; i++) {
      messages[i] = new char[BUFFER_SIZE]();
      snprintf(messages[i], BUFFER_SIZE, "Message %d", i + 1);
      MPI_Isend(messages[i], strlen(messages[i]) + 1, MPI_CHAR, i + 1, 99,
                MPI_COMM_WORLD, &requests[i]);
    }
    int flag;
    MPI_Testall(size - 1, requests.get(), &flag, statuses.get());
    if (!flag) {
      printf("Some of the %d sends have not completed yet.\n", size - 1);
      MPI_Waitall(size - 1, requests.get(), statuses.get());
    }
    printf("All %d sends have completed.\n", size - 1);
    for (int i = 0; i < size - 1; i++) { delete messages[i]; }
  } else {
    char message[BUFFER_SIZE];
    MPI_Request request;
    MPI_Status status;
    MPI_Irecv(message, BUFFER_SIZE, MPI_CHAR, 0, 99, MPI_COMM_WORLD, &request);
    MPI_Wait(&request, &status);
    printf("Process %d received '%s'.\n", myrank, message);
  }
  MPI_Finalize();
  return 0;
}
