/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <errno.h>
#include <pthread.h>
#include <stdint.h>
#include <stdio.h>

#define NTHREADS 2

pthread_barrier_t barrier;

void *thread(void *args) {
  int ret = pthread_barrier_wait(&barrier);
  printf("Returned %d from %ld.\n", ret, reinterpret_cast<int64_t>(args));
  return NULL;
}

int main() {
  pthread_t tids[NTHREADS];
  assert(pthread_barrier_wait(&barrier) == EINVAL);
  assert(pthread_barrier_init(&barrier, NULL, 0) == EINVAL);

  assert(pthread_barrier_init(&barrier, NULL, NTHREADS) == 0);
  assert(pthread_barrier_init(&barrier, NULL, NTHREADS) == EBUSY);
  for (int i = 0; i < NTHREADS; i++) {
    assert(pthread_create(
        &tids[i], NULL, thread, reinterpret_cast<void *>(i)) == 0);
  }
  for (int i = 0; i < NTHREADS; i++) {
    assert(pthread_join(tids[i], NULL) == 0);
  }

  assert(pthread_create(
      &tids[0], NULL, thread, reinterpret_cast<void *>(0)) == 0);
  assert(pthread_barrier_destroy(&barrier) == EBUSY);
  assert(pthread_create(
      &tids[1], NULL, thread, reinterpret_cast<void *>(1)) == 0);
  for (int i = 0; i < NTHREADS; i++) {
    assert(pthread_join(tids[i], NULL) == 0);
  }
  assert(pthread_barrier_destroy(&barrier) == 0);
  assert(pthread_barrier_destroy(&barrier) == EINVAL);
  return 0;
}
