/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <errno.h>
#include <pthread.h>

pthread_cond_t cond;
pthread_mutex_t mutex;

void *signal_thread(void * /* args */) {
  assert(pthread_mutex_lock(&mutex) == 0);
  assert(pthread_cond_destroy(&cond) == EBUSY);
  assert(pthread_cond_signal(&cond) == 0);
  assert(pthread_mutex_unlock(&mutex) == 0);
  return NULL;
}

void *broadcast_thread(void * /* args */) {
  assert(pthread_mutex_lock(&mutex) == 0);
  assert(pthread_cond_destroy(&cond) == EBUSY);
  assert(pthread_cond_broadcast(&cond) == 0);
  assert(pthread_mutex_unlock(&mutex) == 0);
  return NULL;
}

void *wait_thread(void * /* args */) {
  assert(pthread_cond_wait(&cond, &mutex) == EPERM);
  return NULL;
}

int main() {
  pthread_t tid;
  // assert(pthread_cond_broadcast(&cond) == EINVAL);
  // assert(pthread_cond_signal(&cond) == EINVAL);
  // assert(pthread_cond_wait(&cond, &mutex) == EINVAL);

  assert(pthread_cond_init(&cond, NULL) == 0);
  assert(pthread_cond_wait(&cond, &mutex) == EINVAL);
  assert(pthread_cond_init(&cond, NULL) == EBUSY);
  assert(pthread_mutex_init(&mutex, NULL) == 0);
  assert(pthread_cond_wait(&cond, &mutex) == EPERM);

  assert(pthread_mutex_lock(&mutex) == 0);
  assert(pthread_create(&tid, NULL, signal_thread, NULL) == 0);
  assert(pthread_cond_wait(&cond, &mutex) == 0);
  assert(pthread_join(tid, NULL) == 0);
  assert(pthread_create(&tid, NULL, broadcast_thread, NULL) == 0);
  assert(pthread_cond_wait(&cond, &mutex) == 0);
  assert(pthread_join(tid, NULL) == 0);
  assert(pthread_create(&tid, NULL, wait_thread, NULL) == 0);
  assert(pthread_join(tid, NULL) == 0);
  assert(pthread_mutex_unlock(&mutex) == 0);

  assert(pthread_mutex_destroy(&mutex) == 0);
  assert(pthread_cond_destroy(&cond) == 0);
  assert(pthread_cond_destroy(&cond) == EINVAL);
  return 0;
}
