/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>

pthread_rwlock_t rwlock;

void *reader_thread(void * /* args */) {
  assert(pthread_rwlock_rdlock(&rwlock) == 0);
  assert(pthread_rwlock_destroy(&rwlock) == EBUSY);
  assert(pthread_rwlock_unlock(&rwlock) == 0);
  return NULL;
}

void *tryreader_thread(void * /* args */) {
  assert(pthread_rwlock_unlock(&rwlock) == EPERM);
  assert(pthread_rwlock_tryrdlock(&rwlock) == EBUSY);
  return NULL;
}

void *trywriter_thread(void * /* args */) {
  assert(pthread_rwlock_unlock(&rwlock) == EPERM);
  assert(pthread_rwlock_trywrlock(&rwlock) == EBUSY);
  return NULL;
}

int main() {
  pthread_t tid;
  // assert(pthread_rwlock_rdlock(&rwlock) == EINVAL);
  // assert(pthread_rwlock_tryrdlock(&rwlock) == EINVAL);
  // assert(pthread_rwlock_wrlock(&rwlock) == EINVAL);
  // assert(pthread_rwlock_trywrlock(&rwlock) == EINVAL);
  // assert(pthread_rwlock_unlock(&rwlock) == EINVAL);

  assert(pthread_rwlock_init(&rwlock, NULL) == 0);
  assert(pthread_rwlock_init(&rwlock, NULL) == EBUSY);
  assert(pthread_rwlock_unlock(&rwlock) == EPERM);

  assert(pthread_rwlock_rdlock(&rwlock) == 0);
  assert(pthread_create(&tid, NULL, reader_thread, NULL) == 0);
  assert(pthread_join(tid, NULL) == 0);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_tryrdlock(&rwlock) == 0);
  assert(pthread_rwlock_tryrdlock(&rwlock) == 0);
  assert(pthread_rwlock_unlock(&rwlock) == 0);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_trywrlock(&rwlock) == 0);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_rdlock(&rwlock) == 0);
  assert(pthread_rwlock_wrlock(&rwlock) == EDEADLK);
  assert(pthread_rwlock_trywrlock(&rwlock) == EBUSY);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_wrlock(&rwlock) == 0);
  assert(pthread_rwlock_rdlock(&rwlock) == EDEADLK);
  assert(pthread_rwlock_tryrdlock(&rwlock) == EBUSY);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_wrlock(&rwlock) == 0);
  assert(pthread_rwlock_wrlock(&rwlock) == EDEADLK);
  assert(pthread_rwlock_trywrlock(&rwlock) == EBUSY);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_wrlock(&rwlock) == 0);
  assert(pthread_create(&tid, NULL, tryreader_thread, NULL) == 0);
  assert(pthread_join(tid, NULL) == 0);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_wrlock(&rwlock) == 0);
  assert(pthread_create(&tid, NULL, trywriter_thread, NULL) == 0);
  assert(pthread_join(tid, NULL) == 0);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_rdlock(&rwlock) == 0);
  assert(pthread_create(&tid, NULL, trywriter_thread, NULL) == 0);
  assert(pthread_join(tid, NULL) == 0);
  assert(pthread_rwlock_unlock(&rwlock) == 0);

  assert(pthread_rwlock_destroy(&rwlock) == 0);
  assert(pthread_rwlock_destroy(&rwlock) == EINVAL);

  return 0;
}
